class InvalidBase(ValueError):
    __module__ = Exception.__module__


class InvalidMode(ValueError):
    __module__ = Exception.__module__


class ImpossibleValue(ValueError):
    __module__ = Exception.__module__
